#if WIN32
#   define _MSC_VER 1
#	define _CRT_SECURE_NO_WARNINGS
#   include <windows.h>
#else
#
#endif

#include <OpenDoor.h>
#include <stdlib.h>
#include <string.h>
#include "inih/ini.h"
#include "stralloc.h"

struct ptr_vector {
	size_t len;
	size_t capacity;
	void **ptrs;
};

void init_ptr_vector(struct ptr_vector *vec) {
	assert(vec != NULL);
	memset(vec, 0, sizeof(*vec));
}

int ptr_vector_append(struct ptr_vector *vec, void *p) {
	assert(vec != NULL);
	if (vec->len == vec->capacity) {
		void **ps;
		size_t oldcap = vec->capacity;
		if (vec->capacity == 0)
			vec->capacity = 1;
		else
			vec->capacity *= 2;
		ps = realloc(vec->ptrs, vec->capacity * sizeof(void *));
		assert(ps != NULL);
		vec->ptrs = ps;
		memset(vec->ptrs + oldcap, 0, (vec->capacity - oldcap) * sizeof(void *));
	}
	vec->ptrs[vec->len] = p;
	++vec->len;
	return 1;
}

size_t ptr_vector_len(struct ptr_vector *vec) {
	assert(vec != NULL);
	return vec->len;
}


void *ptr_vector_get(struct ptr_vector *vec, size_t i) {
	assert(vec != NULL);
	if (i >= vec->len)
		return NULL;
	assert(vec->ptrs != NULL);
	return vec->ptrs[i];
}

void ptr_vector_destroy(struct ptr_vector *vec) {
	int i;
	for (i=0;i<vec->len;i++) {
		free(vec->ptrs[i]);
	}
	free(vec->ptrs);
}

static const struct ptr_vector EMPTY_PTR_VECTOR = {0, 0, NULL};

struct ansi_t {
    char *name;
    char *path;
};

struct gallery_t {
    char *name;
    int ansi_count;
    struct ansi_t **ansis;
};


int gallery_count;
struct gallery_t **galleries;


static int handler(void* user, const char* section, const char* name,
                   const char* value)
{
    int i;

    for (i=0;i<gallery_count;i++) {
        if (strcasecmp(section, galleries[i]->name) == 0) {
            galleries[i]->ansis = (struct ansi_t **)realloc(galleries[i]->ansis, sizeof(struct ansi_t *) * (galleries[i]->ansi_count+1));
            galleries[i]->ansis[galleries[i]->ansi_count] = (struct ansi_t *)malloc(sizeof(struct ansi_t));
            galleries[i]->ansis[galleries[i]->ansi_count]->name = strdup(name);
            galleries[i]->ansis[galleries[i]->ansi_count]->path = strdup(value);
            galleries[i]->ansi_count++;
            return 1;
        }
    }
    if (gallery_count == 0) {
        galleries = (struct gallery_t **)malloc(sizeof(struct gallery_t *));
    } else {
        galleries = (struct gallery_t **)realloc(galleries, sizeof(struct gallery_t *) * (gallery_count + 1));
    }
    galleries[gallery_count] = (struct gallery_t *)malloc(sizeof(struct gallery_t));
    galleries[gallery_count]->name = strdup(section);
    galleries[gallery_count]->ansi_count = 1;
    galleries[gallery_count]->ansis = (struct ansi_t **)malloc(sizeof(struct ansi_t *));
    galleries[gallery_count]->ansis[0] = (struct ansi_t *)malloc(sizeof(struct ansi_t));
    galleries[gallery_count]->ansis[0]->name = strdup(name);
    galleries[gallery_count]->ansis[0]->path = strdup(value);

    gallery_count++;
    return 1;
}

struct character_t {
	char c;
	int fg;
	int bg;
};

void unmangle_ansi(char *body, int len, char **body_out, int *body_len) {
	// count lines
	int line_count = 1;
	int line_at = 1;
	int char_at = 1;
	int fg = 0x07;
	int bg = 0x00;
	int state = 0;
	int save_char_at = 0;
	int save_line_at = 0;
	int params[16];
	int param_count = 0;
	int bold = 0;
	stralloc out = EMPTY_STRALLOC;
	char buffer[1024];
	int buf_at;
	int i, j, k;
	struct character_t ***fake_screen;
	int ansi;
	int tab;
	int pipec;
	
	line_at = 1;
	char_at = 1;

	for (i = 0; i < len; i++) {

		if (state == 0) {
			if (body[i] == 27) {
				state = 1;
				continue;
			} else {
				if (body[i] == '\r') {
					char_at = 1;
					line_at++;
				} else if (body[i] == '\t') {
					char_at += 8;
					while (char_at > 80) {
						line_at++;
						char_at -= 80;
					}
				} else {
					char_at++;
					while (char_at > 80) {
						line_at++;
						char_at -= 80;
					}
				}

				if (line_at > line_count) {
					line_count = line_at;
				}
			}
		} else if (state == 1) {
			if (body[i] == '[') {
				state = 2;
				continue;
			} else {
				state = 0;
				continue;
			}
		} else if (state == 2) {
			param_count = 0;
			for (j = 0; j < 16; j++) {
				params[j] = 0;
			}
			state = 3;
		}
		if (state == 3) {
			if (body[i] == ';') {
				if (param_count < 15) {
					param_count++;
				}
				continue;
			} else if (body[i] >= '0' && body[i] <= '9') {
				if (!param_count) param_count = 1;
				params[param_count - 1] = params[param_count - 1] * 10 + (body[i] - '0');
				continue;
			} else {
				state = 4;
			}
		}

		if (state == 4) {
			switch (body[i]) {
				case 'H':
				case 'f':
					if (params[0]) params[0]--;
					if (params[1]) params[1]--;
					line_at = params[0] + 1;
					char_at = params[1] + 1;

					if (char_at > 80) {
						char_at = 80;
					}

					if (line_at > line_count) {
						line_count = line_at;
					}

					state = 0;
					break;
				case 'A':
					if (param_count > 0) {
						line_at = line_at - params[0];
					} else {
						line_at--;
					}
					if (line_at < 1) {
						line_at = 1;
					}
					state = 0;
					break;
				case 'B':
					if (param_count > 0) {
						line_at = line_at + params[0];
					} else {
						line_at++;
					}
					if (line_at > line_count) {
						line_count = line_at;
					}
					state = 0;
					break;
				case 'C':
					if (param_count > 0) {
						char_at = char_at + params[0];
					} else {
						char_at++;
					}
					if (char_at > 80) {
						char_at = 80;
					}
					state = 0;
					break;
				case 'D':
					if (param_count > 0) {
						char_at = char_at - params[0];
					} else {
						char_at--;
					}
					if (char_at < 1) {
						char_at = 1;
					}
					state = 0;
					break;
				case 's':
					save_char_at = char_at;
					save_line_at = line_at;
					state = 0;
					break;
				case 'u':
					char_at = save_char_at;
					line_at = save_line_at;
					state = 0;
					break;
				default:
					state = 0;
					break;
			}
		}
	}

	fake_screen = (struct character_t ***)malloc(sizeof(struct character_t **) * line_count);
	for (i = 0; i < line_count; i++) {
		fake_screen[i] = (struct character_t **)malloc(sizeof(struct character_t *) * 80);
		for (j = 0; j < 80; j++) {
			fake_screen[i][j] = (struct character_t *)malloc(sizeof(struct character_t));
			fake_screen[i][j]->c = ' ';
			fake_screen[i][j]->fg = fg;
			fake_screen[i][j]->bg = bg;
		}
	}

	line_at = 1;
	char_at = 1;

	for (i = 0; i < len; i++) {

		if (state == 0) {
			if (body[i] == 27) {
				state = 1;
				continue;
			} else {
				if (body[i] == '\r') {
					char_at = 1;
					line_at++;
				} else if (body[i] == '\t') {
					for (tab = 0; tab < 8; tab++) {
						if (line_at > line_count) line_at = line_count;
						fake_screen[line_at - 1][char_at - 1]->c = ' ';
						fake_screen[line_at - 1][char_at - 1]->fg = fg;
						fake_screen[line_at - 1][char_at - 1]->bg = bg;
						char_at++;
						while (char_at > 80) {
							line_at++;
							char_at -= 80;
						}
					}
				} else {
					if (line_at > line_count) line_at = line_count;
					fake_screen[line_at - 1][char_at - 1]->c = body[i];
					fake_screen[line_at - 1][char_at - 1]->fg = fg;
					fake_screen[line_at - 1][char_at - 1]->bg = bg;
					char_at++;
					while (char_at > 80) {
						line_at++;
						char_at -= 80;
					}
				}
			}
		} else if (state == 1) {
			if (body[i] == '[') {
				state = 2;
				continue;
			} else {
				state = 0;
				continue;
			}
		} else if (state == 2) {
			param_count = 0;
			for (j = 0; j < 16; j++) {
				params[j] = 0;
			}
			state = 3;
		}
		if (state == 3) {
			if (body[i] == ';') {
				if (param_count < 15) {
					param_count++;
				}
				continue;
			} else if (body[i] >= '0' && body[i] <= '9') {
				if (!param_count) param_count = 1;
				params[param_count - 1] = params[param_count - 1] * 10 + (body[i] - '0');
				continue;
			} else {
				state = 4;
			}
		}

		if (state == 4) {
			switch (body[i]) {
				case 'H':
				case 'f':
					if (params[0]) params[0]--;
					if (params[1]) params[1]--;
					line_at = params[0] + 1;
					char_at = params[1] + 1;
					state = 0;
					break;
				case 'A':
					if (param_count > 0) {
						line_at = line_at - params[0];
					} else {
						line_at--;
					}
					if (line_at < 1) {
						line_at = 1;
					}
					state = 0;
					break;
				case 'B':
					if (param_count > 0) {
						line_at = line_at + params[0];
					} else {
						line_at++;
					}
					if (line_at > line_count) {
						line_at = line_count;
					}
					state = 0;
					break;
				case 'C':
					if (param_count > 0) {
						char_at = char_at + params[0];
					} else {
						char_at++;
					}
					if (char_at > 80) {
						char_at = 80;
					}
					state = 0;
					break;
				case 'D':
					if (param_count > 0) {
						char_at = char_at - params[0];
					} else {
						char_at--;
					}
					if (char_at < 1) {
						char_at = 1;
					}
					state = 0;
					break;
				case 's':
					save_char_at = char_at;
					save_line_at = line_at;
					state = 0;
					break;
				case 'u':
					char_at = save_char_at;
					line_at = save_line_at;
					state = 0;
					break;
				case 'm':
					for (j = 0; j < param_count; j++) {
						switch (params[j]) {
							case 0:
								fg = 0x07;
								bg = 0x00;
								bold = 0;
								break;
							case 1:
								bold = 1;
								if (fg < 0x08) {
									fg += 0x08;
								}
								break;
							case 2:
								bold = 0;
								if (fg > 0x07) {
									fg -= 0x08;
								}
								break;
							case 30:
								if (bold) {
									fg = 0x08;
								} else {
									fg = 0x00;
								}
								break;
							case 31:
								if (bold) {
									fg = 0x0C;
								} else {
									fg = 0x04;
								}
								break;
							case 32:
								if (bold) {
									fg = 0x0A;
								} else {
									fg = 0x02;
								}
								break;
							case 33:
								if (bold) {
									fg = 0x0E;
								} else {
									fg = 0x06;
								}
								break;
							case 34:
								if (bold) {
									fg = 0x09;
								} else {
									fg = 0x01;
								}
								break;
							case 35:
								if (bold) {
									fg = 0x0D;
								} else {
									fg = 0x05;
								}
								break;
							case 36:
								if (bold) {
									fg = 0x0B;
								} else {
									fg = 0x03;
								}
								break;
							case 37:
								if (bold) {
									fg = 0x0F;
								} else {
									fg = 0x07;
								}
								break;
							case 40:
								bg = 0x00;
								break;
							case 41:
								bg = 0x04;
								break;
							case 42:
								bg = 0x02;
								break;
							case 43:
								bg = 0x06;
								break;
							case 44:
								bg = 0x01;
								break;
							case 45:
								bg = 0x05;
								break;
							case 46:
								bg = 0x03;
								break;
							case 47:
								bg = 0x07;
								break;
						}
					}
					state = 0;
					break;
				case 'K':
					if (params[0] == 0) {
						for (k = char_at - 1; k < 80; k++) {
							fake_screen[line_at - 1][k]->c = ' ';
							fake_screen[line_at - 1][k]->fg = fg;
							fake_screen[line_at - 1][k]->bg = bg;
						}
					} else if (params[0] == 1) {
						for (k = 0; k < char_at; k++) {
							fake_screen[line_at - 1][k]->c = ' ';
							fake_screen[line_at - 1][k]->fg = fg;
							fake_screen[line_at - 1][k]->bg = bg;
						}
					} else if (params[0] == 2) {
						for (k = 0; k < 80; k++) {
							fake_screen[line_at - 1][k]->c = ' ';
							fake_screen[line_at - 1][k]->fg = fg;
							fake_screen[line_at - 1][k]->bg = bg;
						}
					}
					state = 0;
					break;
				case 'J':
					if (params[0] == 0) {
						for (k = char_at - 1; k < 80; k++) {
							fake_screen[line_at - 1][k]->c = ' ';
							fake_screen[line_at - 1][k]->fg = fg;
							fake_screen[line_at - 1][k]->bg = bg;
						}

						for (k = line_at; k < line_count; k++) {
							for (j = 0; j < 80; j++) {
								fake_screen[k][j]->c = ' ';
								fake_screen[k][j]->fg = fg;
								fake_screen[k][j]->bg = bg;
							}
						}
					} else if (params[0] == 1) {
						for (k = 0; k < char_at; k++) {
							fake_screen[line_at - 1][k]->c = ' ';
							fake_screen[line_at - 1][k]->fg = fg;
							fake_screen[line_at - 1][k]->bg = bg;
						}

						for (k = line_at - 2; k >= 0; k--) {
							for (j = 0; j < 80; j++) {
								fake_screen[k][j]->c = ' ';
								fake_screen[k][j]->fg = fg;
								fake_screen[k][j]->bg = bg;
							}
						}
					} else if (params[0] == 2) {
						for (k = 0; k < line_count; k++) {
							for (j = 0; j < 80; j++) {
								fake_screen[k][j]->c = ' ';
								fake_screen[k][j]->fg = fg;
								fake_screen[k][j]->bg = bg;
							}
						}
					}
					state = 0;
					break;
				default:
					// bad ansi
					state = 0;
					break;
			}
		}
	}

	fg = 0x07;
	bg = 0x00;

	for (i = 0; i < line_count; i++) {
		buf_at = 0;
		for (j = 0; j < 79; j++) {
			if (fake_screen[i][j]->fg != fg || fake_screen[i][j]->bg != bg) {
				buffer[buf_at++] = 27;
				buffer[buf_at++] = '[';
				fg = fake_screen[i][j]->fg;
				if (fg < 0x08) {
					buffer[buf_at++] = '0';
					buffer[buf_at++] = ';';
					buffer[buf_at++] = '3';
					switch (fg) {
						case 0x00:
							buffer[buf_at++] = '0';
							break;
						case 0x04:
							buffer[buf_at++] = '1';
							break;
						case 0x02:
							buffer[buf_at++] = '2';
							break;
						case 0x06:
							buffer[buf_at++] = '3';
							break;
						case 0x01:
							buffer[buf_at++] = '4';
							break;
						case 0x05:
							buffer[buf_at++] = '5';
							break;
						case 0x03:
							buffer[buf_at++] = '6';
							break;
						case 0x07:
							buffer[buf_at++] = '7';
							break;
					}
				} else {
					buffer[buf_at++] = '1';
					buffer[buf_at++] = ';';
					buffer[buf_at++] = '3';
					switch (fg) {
						case 0x08:
							buffer[buf_at++] = '0';
							break;
						case 0x0C:
							buffer[buf_at++] = '1';
							break;
						case 0x0A:
							buffer[buf_at++] = '2';
							break;
						case 0x0E:
							buffer[buf_at++] = '3';
							break;
						case 0x09:
							buffer[buf_at++] = '4';
							break;
						case 0x0D:
							buffer[buf_at++] = '5';
							break;
						case 0x0B:
							buffer[buf_at++] = '6';
							break;
						case 0x0F:
							buffer[buf_at++] = '7';
							break;
					}
				}

				bg = fake_screen[i][j]->bg;
				buffer[buf_at++] = ';';
				buffer[buf_at++] = '4';
				switch (bg) {
					case 0x00:
						buffer[buf_at++] = '0';
						break;
					case 0x04:
						buffer[buf_at++] = '1';
						break;
					case 0x02:
						buffer[buf_at++] = '2';
						break;
					case 0x06:
						buffer[buf_at++] = '3';
						break;
					case 0x01:
						buffer[buf_at++] = '4';
						break;
					case 0x05:
						buffer[buf_at++] = '5';
						break;
					case 0x03:
						buffer[buf_at++] = '6';
						break;
					case 0x07:
						buffer[buf_at++] = '7';
						break;
				}
				buffer[buf_at++] = 'm';
			}
			buffer[buf_at++] = fake_screen[i][j]->c;
		}

		while (buf_at > 0 && buffer[buf_at - 1] == ' ') {
			buf_at--;
		}

		buffer[buf_at++] = '\r';
		stralloc_catb(&out, buffer, buf_at);
	}

	for (i = 0; i < line_count; i++) {
		for (j = 0; j < 80; j++) {
			free(fake_screen[i][j]);
		}
		free(fake_screen[i]);
	}
	free(fake_screen);

	while (out.s[out.len - 2] == '\r') {
		out.len--;
	}

	*body_out = out.s;
	*body_len = out.len;
}

void view_ansi(char *ansi_name, char *ansi_path) {
	FILE *fptr;
	char c;
	int blen;
	stralloc buffer = EMPTY_STRALLOC;
	char *body;
	int len;
	int start_line;
	int z;
	int chars;
	struct ptr_vector msg_lines = EMPTY_PTR_VECTOR;
	int ansi;
	int y, z2, j;
	int position = 0;
	int should_break = 0;
	tODInputEvent ch;
	
	fptr = fopen(ansi_path, "rb");
	
	if (!fptr) {
		return;
	}
	
	len = 0;
	blen = 256;
	
	c = fgetc(fptr);
	
	while (!feof(fptr) && c != 0x1a) {
		if (c != '\n') {
			stralloc_append1(&buffer, c);
		}
		c = fgetc(fptr);
	}
	
	unmangle_ansi(buffer.s, buffer.len, &body, &blen);
	free(buffer.s);

	start_line = 0;
	chars = 0;
	z2 = blen;
	// count the number of lines...
	for (z = 0; z < z2; z++) {
		if (body[z] == '\r' || chars == 80) {
			char *msg_line = (char *)malloc(z - start_line + 1);
			ptr_vector_append(&msg_lines, msg_line);
			if (z == start_line) {
				msg_line[0] = '\0';
			} else {
				strncpy(msg_line, &body[start_line], z - start_line + 1);
				msg_line[z - start_line] = '\0';
			}
			if (body[z] == '\r') {
				start_line = z + 1;
			} else {
				start_line = z;
			}
			chars = 0;
		} else {
			if (body[z] == 27) {
				ansi = z;
				while (strchr("ABCDEFGHIGJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz", body[z]) == NULL)
					z++;
				if (body[z] == 'm') {
					// do nothing
				} else {
					y = ansi;
					for (j = z + 1; j < z2; j++) {
						body[y] = body[j];
						y++;
					}
					z2 = z2 - (z2 - y);
					z = ansi - 1;
				}
			} else {
				chars++;
			}
		}
	}
	
	free(body);
	
	od_clr_scr();
	position = 0;
    od_set_cursor(1, 1);
    od_set_color(L_YELLOW, D_BLUE);
    od_printf("%s", ansi_name);
    od_clr_line();

    od_set_cursor(24, 1);
    od_set_color(L_YELLOW, D_BLUE);
    od_printf("Up/Down to Scroll, Q to Quit");
    od_clr_line();
	od_set_color(D_GREY, D_BLACK);
	
	while(!should_break) {
		for (z = position; z < ptr_vector_len(&msg_lines); z++) {
			od_set_cursor(z - position + 2, 1);
			od_printf("%s", ptr_vector_get(&msg_lines, z));
			od_clr_line();
			if (z - position >= 21) {
				break;
			}
		}
		od_get_input(&ch, OD_NO_TIMEOUT, GETIN_NORMAL);
		if (ch.EventType == EVENT_EXTENDED_KEY) {
            switch (ch.chKeyPress) {
            case OD_KEY_DOWN:
				if (ptr_vector_len(&msg_lines) > 22) {
					if (position < ptr_vector_len(&msg_lines) - 22) {
						position++;
					}
				}
				break;
            case OD_KEY_UP:
				if (position > 0) {
					position--;
				}
                break;
            }
		} else if (ch.EventType == EVENT_CHARACTER) {
            switch (ch.chKeyPress) {
            case 'q':
            case 'Q':
				ptr_vector_destroy(&msg_lines);
                return;
			}
		}
		
	}
}

int view_gallery(int galleryno) {
	int i;
	int position = 0;
	int start = 0;
	int redraw = 1;
    tODInputEvent ch;

    while (1) {
        if (redraw) {
            od_clr_scr();
            od_set_cursor(1, 1);
            od_set_color(L_YELLOW, D_BLUE);
            od_printf("%s", galleries[galleryno]->name);
            od_clr_line();

            od_set_cursor(24, 1);
            od_set_color(L_YELLOW, D_BLUE);
            od_printf("Up/Down to Select, Enter to view, Q to go Back");
            od_clr_line();

            od_set_color(L_WHITE, D_BLACK);

            for (i=start;i < start + 22 && i < galleries[galleryno]->ansi_count; i++) {
                od_set_cursor(i - start + 2, 1);
                if (i == position) {
                    od_set_color(D_BLACK, D_CYAN);
                }
                od_printf("%s", galleries[galleryno]->ansis[i]->name);
                od_clr_line();
                if (i == position) {
                    od_set_color(L_WHITE, D_BLACK);
                }
            }
            redraw = 0;
        }

        od_get_input(&ch, OD_NO_TIMEOUT, GETIN_NORMAL);
		if (ch.EventType == EVENT_EXTENDED_KEY) {
            switch (ch.chKeyPress) {
            case OD_KEY_DOWN:
				if (position + 1 < galleries[galleryno]->ansi_count) {
					if (position + 1 >= start + 22) {
						redraw = 1;
						start += 22;
						if (start >= galleries[galleryno]->ansi_count) {
							start = galleries[galleryno]->ansi_count - 22;
							if (start < 0) {
								start = 0;
							}
		
						}                     
						position++;
					} else {
						position++;
						if (position == galleries[galleryno]->ansi_count) {
							position--;
						}
					}
				   
					if (!redraw  && galleries[galleryno]->ansi_count > 1) {
						od_set_cursor(position - start + 1, 1);
						od_printf("%s", galleries[galleryno]->ansis[position -1]->name);
						od_clr_line();
						od_set_color(D_BLACK, D_CYAN);
						od_set_cursor(position - start + 2, 1);
						od_printf("%s", galleries[galleryno]->ansis[position]->name);
						od_clr_line();
						od_set_color(L_WHITE, D_BLACK);
					}
				}
                break;
            case OD_KEY_UP:
				if (position -1 >= 0) {
					if (position - 1 < start) {
						redraw = 1;
						start -= 22;
						if (start < 0) {
							start = 0;
						} 
						position--;
					} else {
						position--;
					}


					if (!redraw && galleries[galleryno]->ansi_count > 1) {
						od_set_cursor(position - start + 3, 1);
						od_printf("%s", galleries[galleryno]->ansis[position +1]->name);
						od_clr_line();
						od_set_color(D_BLACK, D_CYAN);
						od_set_cursor(position - start + 2, 1);
						od_printf("%s", galleries[galleryno]->ansis[position]->name);
						od_clr_line();
						od_set_color(L_WHITE, D_BLACK);
					}
				}
                break;
            }
		} else if (ch.EventType == EVENT_CHARACTER) {
            switch (ch.chKeyPress) {
            case 'q':
            case 'Q':
                return 0;
            case 13:
				view_ansi(galleries[galleryno]->ansis[position]->name, galleries[galleryno]->ansis[position]->path);
                redraw = 1;
                break;
            }
		}

    }
    return 0;
}

#if _MSC_VER
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine, int nCmdShow)
{
#else
int main(int argc, char **argv)
{
#endif
	int done = 0;
	int i;
	int position = 0;
	int start = 0;
	int redraw = 1;
	gallery_count = 0;
	tODInputEvent ch;
#if _MSC_VER
	od_parse_cmd_line(lpszCmdLine);
#else
	od_parse_cmd_line(argc, argv);
#endif

	od_init();

	od_control_get()->od_page_pausing = 0;
	od_control_get()->od_color_delimiter = 0;
    if (ini_parse("gallery.ini", handler, NULL) <0) {
		od_printf("`bright red`Unable to open gallery.ini\r\n");
		od_exit(-1, FALSE);
		return -1;
	}

    while (!done) {
        if (redraw) {
            od_clr_scr();
            od_set_cursor(1, 1);
            od_set_color(L_YELLOW, D_BLUE);
            od_printf("Ansi Gallery v0.2");
            od_clr_line();

            od_set_cursor(24, 1);
            od_set_color(L_YELLOW, D_BLUE);
            od_printf("Up/Down to Select, Enter to view, Q to Quit");
            od_clr_line();

            od_set_color(L_WHITE, D_BLACK);

            for (i=start;i < start + 22 && i < gallery_count; i++) {
                od_set_cursor(i - start + 2, 1);
                if (i == position) {
                    od_set_color(D_BLACK, D_CYAN);
                }
                od_printf("%s", galleries[i]->name);
                od_clr_line();
                if (i == position) {
                    od_set_color(L_WHITE, D_BLACK);
                }
            }
            redraw = 0;
        }
        od_get_input(&ch, OD_NO_TIMEOUT, GETIN_NORMAL);
		if (ch.EventType == EVENT_EXTENDED_KEY) {
            switch (ch.chKeyPress) {
            case OD_KEY_DOWN:
				if (position + 1 < gallery_count) {
					if (position + 1 >= start + 22) {
						redraw = 1;
						start += 22;
						if (start >= gallery_count) {
							start = gallery_count - 22;
							if (start < 0) {
								start = 0;
							}
						}
						position++;
					} else {
						position++;
						if (position == gallery_count) {
							position--;
						}
					}

					if (!redraw && gallery_count > 1) {
						od_set_cursor(position - start + 1, 1);
						od_printf("%s", galleries[position -1]->name);
						od_clr_line();
						od_set_color(D_BLACK, D_CYAN);
						od_set_cursor(position - start + 2, 1);
						od_printf("%s", galleries[position]->name);
						od_clr_line();
						od_set_color(L_WHITE, D_BLACK);
					}
				}
                break;
            case OD_KEY_UP:
				if (position - 1 >= 0) {
					if (position - 1 < start) {
						redraw = 1;
						start -= 22;
						if (start < 0) {
							start = 0;
						} 
						position--;
					} else {
						position--;
					}

					if (position < 0) {
						position = 0;
					} 
					if (!redraw && gallery_count > 1) {
						od_set_cursor(position - start + 3, 1);
						od_printf("%s", galleries[position+1]->name);
						od_clr_line();
						od_set_color(D_BLACK, D_CYAN);
						od_set_cursor(position - start + 2, 1);
						od_printf("%s", galleries[position]->name);
						od_clr_line();
						od_set_color(L_WHITE, D_BLACK);
					}
				}
                break;
            }
		} else if (ch.EventType == EVENT_CHARACTER) {
            switch (ch.chKeyPress) {
            case 'q':
            case 'Q':
                done = 1;
                break;
            case 13:
                if (view_gallery(position) != 0) {
                    done = 1;
                }
                redraw = 1;
                break;
            }
		}

    }

	od_exit(0, FALSE);

	return 0;
}
